#include "game2048.hpp"
#include <iostream>  

Game2048::Game2048() { // Start my game
    score = 0; // Points start at 0
    for (int r = 0; r < 4; r++) { 
        for (int c = 0; c < 4; c++) { 
            board[r][c] = 0; 
        }
    }
    board[0][0] = 2; // Put a 2 in top left
    board[0][1] = 2; // Put another 2 next to it
}

void Game2048::show() { // Show my tiles in boxes
    for (int r = 0; r < 4; r++) { // Look at each row
        std::cout << "+---+---+---+---+"; // Draw top of boxes
        std::cout << "\n"; // Go to new line
        for (int c = 0; c < 4; c++) { 
            std::cout << "| "; // Start a box
            if (board[r][c] == 0) { 
                std::cout << " "; // Just a space
            } else { 
                std::cout << board[r][c]; // Show the number
            }
            std::cout << " "; 
        }
        std::cout << "|"; 
        std::cout << "\n"; // Go to new line
    }
    std::cout << "+---+---+---+---+"; 
    std::cout << "\n"; // Go to new line
    std::cout << "Points: " << score << "\n"; // Show my points
    std::cout << "\n"; // Extra space
}

void Game2048::addTile() { // Add a new number
    for (int r = 0; r < 4; r++) { // Check all rows
        for (int c = 0; c < 4; c++) { // Check all spots
            if (board[r][c] == 0) { // If it’s empty
                if (r < 2) { // Top rows get 2
                    board[r][c] = 2; // Put a 2
                } else { // Bottom rows get 4
                    board[r][c] = 4; // Put a 4
                }
                return; // Stop after one
            }
        }
    }
}

bool Game2048::canMove() { // Can I slide tiles?
    for (int r = 0; r < 4; r++) { // Check every row
        for (int c = 0; c < 4; c++) { // Check every spot
            if (board[r][c] == 0) { // Empty spot?
                return true; // Yes, I can move
            }
            if (c < 3) { // Not at end of row
                if (board[r][c] == board[r][c + 1]) { // Same numbers next to each other
                    return true; // I can squish them
                }
            }
            if (r < 3) { // Not at bottom
                if (board[r][c] == board[r + 1][c]) { // Same numbers up and down
                    return true; // I can squish these
                }
            }
        }
    }
    return false; // No sliding possible
}

void Game2048::moveLeft() { // Slide tiles left
    for (int r = 0; r < 4; r++) { // Do every row
        for (int c = 0; c < 3; c++) { // Look at spots 0 to 2
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r][c + 1]; // Move next number here
                board[r][c + 1] = 0; // Empty the next spot
            }
        }
        for (int c = 0; c < 3; c++) { // Squish same numbers
            if (board[r][c] != 0 && board[r][c] == board[r][c + 1]) { // If they’re the same
                board[r][c] = board[r][c] * 2; // Make it bigger
                score = score + board[r][c]; // Add points
                board[r][c + 1] = 0; // Empty the next one
            }
        }
        for (int c = 0; c < 3; c++) { // Slide again to fill gaps
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r][c + 1]; // Move next number here
                board[r][c + 1] = 0; // Empty the next spot
            }
        }
    }
    if (canMove()) addTile(); // Add a number if I can slide
}

void Game2048::moveRight() { // Slide tiles right
    for (int r = 0; r < 4; r++) { // Do every row
        for (int c = 3; c > 0; c--) { // Look at spots 3 to 1
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r][c - 1]; // Move left number here
                board[r][c - 1] = 0; // Empty the left spot
            }
        }
        for (int c = 3; c > 0; c--) { // Squish same numbers
            if (board[r][c] != 0 && board[r][c] == board[r][c - 1]) { // If they’re the same
                board[r][c] = board[r][c] * 2; // Make it bigger
                score = score + board[r][c]; // Add points
                board[r][c - 1] = 0; // Empty the left one
            }
        }
        for (int c = 3; c > 0; c--) { // Slide again to fill gaps
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r][c - 1]; // Move left number here
                board[r][c - 1] = 0; // Empty the left spot
            }
        }
    }
    if (canMove()) addTile(); // Add a number if I can slide
}

void Game2048::moveUp() { // Slide tiles up
    for (int c = 0; c < 4; c++) { // Do every column
        for (int r = 0; r < 3; r++) { // Look at rows 0 to 2
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r + 1][c]; // Move down number up
                board[r + 1][c] = 0; // Empty the down spot
            }
        }
        for (int r = 0; r < 3; r++) { // Squish same numbers
            if (board[r][c] != 0 && board[r][c] == board[r + 1][c]) { // If they’re the same
                board[r][c] = board[r][c] * 2; // Make it bigger
                score = score + board[r][c]; // Add points
                board[r + 1][c] = 0; // Empty the next one
            }
        }
        for (int r = 0; r < 3; r++) { // Slide again to fill gaps
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r + 1][c]; // Move down number up
                board[r + 1][c] = 0; // Empty the down spot
            }
        }
    }
    if (canMove()) addTile(); // Add a number if I can slide
}

void Game2048::moveDown() { // Slide tiles down
    for (int c = 0; c < 4; c++) { // Do every column
        for (int r = 3; r > 0; r--) { // Look at rows 3 to 1
            if (board[r][c] == 0) { // If spot is empty
                board[r][c] = board[r - 1][c]; // Move up number down
                board[r - 1][c] = 0; // Empty the up spot
            }
        }
        for (int r = 3; r > 0; r--) { // Squish same numbers
            if (board[r][c] != 0 && board[r][c] == board[r - 1][c]) { 
                board[r][c] = board[r][c] * 2; 
                score = score + board[r][c]; 
                board[r - 1][c] = 0; 
            }
        }
        for (int r = 3; r > 0; r--) { 
            if (board[r][c] == 0) { 
                board[r][c] = board[r - 1][c]; // Move up number down
                board[r - 1][c] = 0; 
            }
        }
    }
    if (canMove()) addTile(); // Add a number if I can slide
}

void Game2048::play() { // Play my game
    while (true) { 
        show(); // Show my tiles
        char key; // What I press
        std::cin >> key; // Get my letter
        if (key == 'Q') { // If I press Q
            std::cout << "Game Over! Points: " << score << "\n"; // Go to new line
            break; // Stop playing
        }
        if (key == 'A') { // If I press A
            moveLeft(); // Slide left
        }
        if (key == 'D') { // If I press D
            moveRight(); // Slide right
        }
        if (key == 'W') { // If I press W
            moveUp(); // Slide up
        }
        if (key == 'S') { // If I press S
            moveDown(); // Slide down
        }
        if (!canMove()) { // If I can’t slide anymore
            show(); // Show last tiles
            std::cout << "Game Over! Points: " << score << "\n"; // Go to new line
            break; // Stop playing
        }
    }
}